%define anolis_release 1

# Don't add -Wl,-dT,<build dir>
%undefine _package_note_flags

# OCaml has a bytecode backend that works on anything with a C
# compiler, and a native code backend available on a subset of
# architectures.  A further subset of architectures support native
# dynamic linking.

%ifarch %{ocaml_native_compiler}
%global native_compiler 1
%else
%global native_compiler 0
%endif

%ifarch %{ocaml_natdynlink}
%global natdynlink 1
%else
%global natdynlink 0
%endif

Name:           ocaml
Version:        4.14.0
Release:        %{anolis_release}%{?dist}
Summary:        OCaml compiler and programming environment
License:        QPL and (LGPLv2+ with exceptions)
URL:            https://www.ocaml.org
Source0:        https://github.com/ocaml/ocaml/archive/refs/tags/%{version}.tar.gz

Patch0001:      0001-Do-not-trigger-warning-when-calling-virtual-methods-.patch
Patch0002:      0002-Merge-pull-request-11236-from-Nymphium-missing-since.patch
Patch0003:      0003-misc.h-fix-preprocessor-conditional-on-_MSC_VER.patch
Patch0004:      0004-Changes.patch
Patch0005:      0005-Guard-more-instances-of-undefined-_MSC_VER.patch
Patch0006:      0006-Better-documentation-for-string_of_float-.-11353.patch
Patch0007:      0007-Merge-pull-request-11380-from-damiendoligez-fix-fort.patch
Patch0008:      0008-Refactor-the-initialization-of-bytecode-threading-11.patch
Patch0009:      0009-Merge-pull-request-11397-from-Octachron-tast_mapper_.patch
Patch0010:      0010-Merge-pull-request-11396-from-gasche-fix11392.patch
Patch0011:      0011-Document-limitation-on-caml_callbackN-11409.patch
Patch0012:      0012-Stop-calling-ranlib-on-created-installed-libraries-1.patch
Patch0013:      0013-tests-lib-bigarray-2-has-gfortran.sh-don-t-print-any.patch
Patch0014:      0014-Merge-pull-request-11417-from-lpw25-fix-virtual-clas.patch
Patch0015:      0015-Do-not-elide-the-whole-module-type-error-message-114.patch
Patch0016:      0016-Merge-pull-request-11373-from-dra27-flexlink-detect.patch
Patch0017:      0017-Merge-pull-request-11468-from-dra27-i686-mingw-ipv6.patch
Patch0018:      0018-More-prudent-deallocation-of-alternate-signal-stack-.patch
Patch0019:      0019-Merge-pull-request-11487-from-purplearmadillo77-fma_.patch
Patch0020:      0020-Fixup-Changes.patch
Patch0021:      0021-Fix-deprecated_mutable-which-couldn-t-be-triggered.-.patch
Patch0022:      0022-Don-t-add-rpaths-to-libraries.patch

BuildRequires:  make
BuildRequires:  git
BuildRequires:  gcc
BuildRequires:  autoconf
BuildRequires:  binutils-devel
BuildRequires:  ncurses-devel
BuildRequires:  gdbm-devel
BuildRequires:  gawk
BuildRequires:  perl-interpreter
BuildRequires:  util-linux
BuildRequires:  /usr/bin/annocheck
BuildRequires:  ocaml-srpm-macros

# ocamlopt runs gcc to link binaries.  Because Anolis includes
# hardening flags automatically, system-rpm-config is also required.
Requires:       gcc
Requires:       system-rpm-config

# Because we pass -c flag to ocaml-find-requires (to avoid circular
# dependencies) we also have to explicitly depend on the right version
# of ocaml-runtime.
Requires:       ocaml-runtime = %{version}-%{release}

# Bundles an MD5 implementation in runtime/caml/md5.h and runtime/md5.c
Provides:       bundled(md5-plumb)

Provides:       ocaml(compiler) = %{version}

%global __ocaml_requires_opts -c -f '%{buildroot}%{_bindir}/ocamlrun %{buildroot}%{_bindir}/ocamlobjinfo.byte'
%global __ocaml_provides_opts -f '%{buildroot}%{_bindir}/ocamlrun %{buildroot}%{_bindir}/ocamlobjinfo.byte'

%description
OCaml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

This package comprises two batch compilers (a fast bytecode compiler
and an optimizing native-code compiler), an interactive toplevel system,
parsing tools (Lex,Yacc), a replay debugger, a documentation generator,
and a comprehensive library.

%package runtime
Summary:        OCaml runtime environment
Requires:       util-linux
Provides:       ocaml(runtime) = %{version}

%description runtime
OCaml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

This package contains the runtime environment needed to run OCaml
bytecode.

%package source
Summary:        Source code for OCaml libraries
Requires:       ocaml = %{version}-%{release}

%description source
Source code for OCaml libraries.

%package ocamldoc
Summary:        Documentation generator for OCaml
Requires:       ocaml = %{version}-%{release}
Provides:       ocamldoc = %{version}

%description ocamldoc
Documentation generator for OCaml.

%package doc
Summary:        Documentation for OCaml
Requires:       ocaml = %{version}-%{release}
Provides:       %{name}-docs = %{version}-%{release}
Obsoletes:      %{name}-docs < %{version}-%{release}

%description doc
OCaml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

This package contains man pages and documentation files for %{name}.

%package compiler-libs
Summary:        Compiler libraries for OCaml
Requires:       ocaml = %{version}-%{release}

%description compiler-libs
OCaml is a high-level, strongly-typed, functional and object-oriented
programming language from the ML family of languages.

This package contains some modules used internally by the OCaml
compilers, useful for the development of some OCaml applications.
Note that this exposes internal details of the OCaml compiler which
may not be portable between versions.

%prep
%autosetup -S git -n %{name}-%{version}
# Patches touch configure.ac, so rebuild it:
autoconf --force

%build
make="%make_build"

# Don't use %%configure macro because it sets --build, --host which
# breaks some incorrect assumptions made by OCaml's configure.ac
#
# We set --libdir to the unusual directory because we want OCaml to
# install its libraries and other files into a subdirectory.
#
# OC_CFLAGS/OC_LDFLAGS control what flags OCaml passes to the linker
# when doing final linking of OCaml binaries.
./configure \
    --prefix=%{_prefix} \
    --sysconfdir=%{_sysconfdir} \
    --mandir=%{_mandir} \
    --libdir=%{_libdir}/ocaml \
    OC_CFLAGS="$CFLAGS" \
    OC_LDFLAGS="$LDFLAGS" \
    %{nil}
$make world
%if %{native_compiler}
$make opt
$make opt.opt
%endif

%check
%ifarch %{ocaml_native_compiler}
# For information only, compile a binary and dump the annocheck data
# from it.  Useful so we know if hardening is being enabled, but don't
# fail because not every hardening feature can be enabled here.
echo 'print_endline "hello, world"' > hello.ml
./ocamlopt.opt -verbose -I stdlib hello.ml -o hello ||:
annocheck -v hello ||:
%endif

make ocamltest
%ifarch %{ocaml_native_compiler}
make ocamltest.opt
%endif
make -j1 tests ||:

%install
%make_install
perl -pi -e "s|^$RPM_BUILD_ROOT||" $RPM_BUILD_ROOT%{_libdir}/ocaml/ld.conf

echo %{version} > $RPM_BUILD_ROOT%{_libdir}/ocaml/anolis-ocaml-release

# Remove the installed documentation.  We will install it using %%doc
rm -rf $RPM_BUILD_ROOT%{_docdir}/ocaml

# Remove this file.  It's only created in certain situations and it's
# unclear why it is created at all.
rm -f $RPM_BUILD_ROOT%{_libdir}/ocaml/eventlog_metadata

%generate_compatibility_deps

%files
%license LICENSE
%{_bindir}/ocaml

%{_bindir}/ocamlcmt
%{abidir}/ocamlcmt-option.list
%{_bindir}/ocamldebug
%{_bindir}/ocamlyacc

# symlink to either .byte or .opt version
%{_bindir}/ocamlc
%{_bindir}/ocamlcp
%{_bindir}/ocamldep
%{_bindir}/ocamllex
%{_bindir}/ocamlmklib
%{_bindir}/ocamlmktop
%{_bindir}/ocamlobjinfo
%{_bindir}/ocamloptp
%{_bindir}/ocamlprof

# bytecode versions
%{_bindir}/ocamlc.byte
%{_bindir}/ocamlcp.byte
%{_bindir}/ocamldep.byte
%{_bindir}/ocamllex.byte
%{_bindir}/ocamlmklib.byte
%{_bindir}/ocamlmktop.byte
%{_bindir}/ocamlobjinfo.byte
%{_bindir}/ocamloptp.byte
%{_bindir}/ocamlprof.byte

%if %{native_compiler}
# native code versions
%{_bindir}/ocamlc.opt
%{_bindir}/ocamlcp.opt
%{_bindir}/ocamldep.opt
%{_bindir}/ocamllex.opt
%{_bindir}/ocamlmklib.opt
%{_bindir}/ocamlmktop.opt
%{_bindir}/ocamlobjinfo.opt
%{_bindir}/ocamloptp.opt
%{_bindir}/ocamlprof.opt
%{abidir}/ocamlc.opt-option.list
%{abidir}/ocamlcp.opt-option.list
%{abidir}/ocamldep.opt-option.list
%{abidir}/ocamllex.opt-option.list
%{abidir}/ocamlmklib.opt-option.list
%{abidir}/ocamlmktop.opt-option.list
%{abidir}/ocamlobjinfo.opt-option.list
%{abidir}/ocamloptp.opt-option.list
%{abidir}/ocamlprof.opt-option.list
%endif

%if %{native_compiler}
%{_bindir}/ocamlopt
%{_bindir}/ocamlopt.byte
%{_bindir}/ocamlopt.opt
%{abidir}/ocamlopt.opt-option.list
%endif

%{_libdir}/ocaml/camlheader
%{_libdir}/ocaml/camlheader_ur
%{_libdir}/ocaml/expunge
%{_libdir}/ocaml/ld.conf
%{_libdir}/ocaml/Makefile.config
%{_libdir}/ocaml/*.a
%if %{natdynlink}
%{_libdir}/ocaml/*.cmxs
%endif
%if %{native_compiler}
%{_libdir}/ocaml/*.cmxa
%{_libdir}/ocaml/*.cmx
%{_libdir}/ocaml/*.o
%{_libdir}/ocaml/libasmrun_shared.so
%{abidir}/libasmrun_shared.dump
%endif
%{_libdir}/ocaml/*.mli
%{_libdir}/ocaml/libcamlrun_shared.so
%{abidir}/libcamlrun_shared.dump
%{_libdir}/ocaml/threads/*.mli
%if %{native_compiler}
%{_libdir}/ocaml/threads/*.a
%{_libdir}/ocaml/threads/*.cmxa
%{_libdir}/ocaml/threads/*.cmx
%endif
%{_libdir}/ocaml/caml

%files runtime
%license LICENSE
%dir %{abidir}
%{_bindir}/ocamlrun
%{_bindir}/ocamlrund
%{_bindir}/ocamlruni
%{abidir}/ocamlrun-option.list
%{abidir}/ocamlrund-option.list
%{abidir}/ocamlruni-option.list
%dir %{_libdir}/ocaml
%{_libdir}/ocaml/*.cmo
%{_libdir}/ocaml/*.cmi
%{_libdir}/ocaml/*.cma
%{_libdir}/ocaml/camlheaderd
%{_libdir}/ocaml/camlheaderi
%{_libdir}/ocaml/stublibs
%dir %{_libdir}/ocaml/threads
%{_libdir}/ocaml/threads/*.cmi
%{_libdir}/ocaml/threads/*.cma
%{_libdir}/ocaml/anolis-ocaml-release
%{abidir}/dll*.dump

%files source
%license LICENSE
%{_libdir}/ocaml/*.ml
%{_libdir}/ocaml/*.cmt*
%{_libdir}/ocaml/*/*.cmt*

%files ocamldoc
%license LICENSE
%doc ocamldoc/Changes.txt
%{_bindir}/ocamldoc*
%{abidir}/ocamldoc*
%{_libdir}/ocaml/ocamldoc

%files doc
%{_mandir}/man1/*
%{_mandir}/man3/*
%doc README.adoc Changes

%files compiler-libs
%license LICENSE
%dir %{_libdir}/ocaml/compiler-libs
%{_libdir}/ocaml/compiler-libs/*.mli
%{_libdir}/ocaml/compiler-libs/*.cmi
%{_libdir}/ocaml/compiler-libs/*.cmo
%{_libdir}/ocaml/compiler-libs/*.cma
%if %{native_compiler}
%{_libdir}/ocaml/compiler-libs/*.a
%{_libdir}/ocaml/compiler-libs/*.cmxa
%{_libdir}/ocaml/compiler-libs/*.cmx
%{_libdir}/ocaml/compiler-libs/*.o
%endif

%changelog
* Mon Mar 20 2023 Yuanhong Peng <yummypeng@linux.alibaba.com> - 4.14.0-1
- Init from upstream
